# encoding: utf-8

from ideabox.policy import _
from plone import api
from plone.supermodel.model import Schema
from plone.tiles import Tile
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope import schema


class IPriorityActionTile(Schema):
    """A tile that displays a listing of content items"""

    title = schema.TextLine(
        title=_(u"Title"), required=True, default=_(u"Priority action")
    )


class LatestPriorityActionTile(Tile):
    template = ViewPageTemplateFile("templates/priority_action.pt")

    def __call__(self):
        return self.template()

    @property
    def title(self):
        return self.data.get("title") or _(u"Priority action")

    def lastcontent(self):
        return api.content.find(
            portal_type="priority_action", sort_on="created", sort_order="reverse"
        )[0]

    def contents(self):
        return api.content.find(
            portal_type="priority_action", sort_on="created", sort_order="reverse"
        )[1:6]

    def folder_projects(self):
        folder = api.content.find(portal_type="Folder", id="projets")
        if len(folder) == 1:
            return folder[0].getURL()
        return False

    @property
    def default_image(self):
        return "{0}/project_default.jpg".format(api.portal.get().absolute_url())



# -*- coding: utf-8 -*-

from plone import api
from plone.app.layout.viewlets import ViewletBase
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getMultiAdapter


class UserMenuViewlet(ViewletBase):

    index = ViewPageTemplateFile("templates/user_menu.pt")

    def user_actions(self):
        context_state = getMultiAdapter(
            (self.context, self.request), name=u"plone_context_state"
        )

        actions = context_state.actions("user_menu")
        return actions

    def user_name(self):
        user = api.user.get_current()
        return user



# -*- coding: utf-8 -*-

from Products.CMFPlone.interfaces import INonInstallable
from collective.taxonomy.exportimport import TaxonomyImportExportAdapter
from collective.taxonomy.factory import registerTaxonomy
from collective.taxonomy.interfaces import ITaxonomy
from eea.facetednavigation.layout.layout import FacetedLayout
from ideabox.policy import _
from plone import api
from plone.dexterity.interfaces import IDexterityFTI
from plone.portlets.constants import CONTEXT_CATEGORY
from plone.portlets.interfaces import ILocalPortletAssignmentManager
from plone.portlets.interfaces import IPortletManager
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryUtility
from zope.i18n import translate
from zope.i18n.interfaces import ITranslationDomain
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory

import os


@implementer(INonInstallable)
class HiddenProfiles(object):
    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return ["ideabox.policy:uninstall"]


def post_install(context):
    """Post install script"""
    # add_taxonomies()
    portal = api.portal.get()
    if portal.get("projets") is None:
        project = api.content.create(
            type="Folder", id="projets", title="Projets", container=portal
        )
        api.content.transition(obj=project, transition="publish")
        add_behavior(
            "Folder",
            "eea.facetednavigation.subtypes.interfaces.IPossibleFacetedNavigable",
        )
        _activate_faceted_navigation(project, True, "/faceted/config/projets.xml")
        project_layout = FacetedLayout(project)
        project_layout.update_layout(layout="faceted-project")
        _disable_portlets(project)
        _publish(project)
    if portal.get("participer") is None:
        participate = api.content.create(
            type="Folder", id="participer", title="Participer", container=portal
        )
        _disable_portlets(participate)
        _publish(participate)
    if portal.get("plus-dinfos") is None:
        infos = api.content.create(
            type="Folder", id="plus-dinfos", title=u"Plus d'infos", container=portal
        )
        _disable_portlets(infos, disabled=("plone.rightcolumn",))
        _publish(infos)

    add_behavior(
        "Collection",
        "eea.facetednavigation.subtypes.interfaces.IPossibleFacetedNavigable",
    )
    if "news" in portal:
        _activate_faceted_navigation(
            portal["news"]["aggregator"], True, "/faceted/config/news.xml"
        )
        _disable_portlets(portal["news"])
        news_layout = FacetedLayout(portal["news"]["aggregator"])
        news_layout.update_layout(layout="faceted-news")

    allowed_sizes = api.portal.get_registry_record("plone.allowed_sizes")
    scales = ("banner 1920:800", "project_faceted 450:300", "evenement 300:300")
    for scale in scales:
        if scale not in allowed_sizes:
            allowed_sizes.append(scale)
    api.portal.set_registry_record("plone.allowed_sizes", allowed_sizes)

    api.portal.set_registry_record(
        "collective.behavior.banner.browser.controlpanel.IBannerSettingsSchema.banner_scale",
        "banner",
    )

    # menu = {
    #     "/": [
    #         {
    #             "navigation_folder": "/projets",
    #             "simple_link": "/projets",
    #             "tab_title": "Projets",
    #             "additional_columns": "",
    #             "condition": "python: True",
    #         },
    #         {
    #             "navigation_folder": "",
    #             "simple_link": "/participer",
    #             "tab_title": "Participer",
    #             "additional_columns": "",
    #             "condition": "python: True",
    #         },
    #         {
    #             "navigation_folder": "/plus-dinfos",
    #             "simple_link": "/plus-dinfos",
    #             "tab_title": "Plus d'infos",
    #             "additional_columns": "",
    #             "condition": "python: True",
    #         },
    #     ]
    # }
    # api.portal.set_registry_record(
    #     "collective.editablemenu.browser.interfaces.IEditableMenuSettings.menu_tabs_json",
    #     json.dumps(menu),
    # )


def add_taxonomies():
    current_lang = api.portal.get_current_language()[:2]

    data_theme = {
        "taxonomy": "theme",
        "field_title": translate(_("Theme"), target_language=current_lang),
        "field_description": "",
        "default_language": "fr",
        "filename": "taxonomy-settings-theme.xml",
    }

    data_district = {
        "taxonomy": "district",
        "field_title": translate(_("District"), target_language=current_lang),
        "field_description": "",
        "default_language": "fr",
        "filename": "taxonomy-settings-district.xml",
    }

    data_iam = {
        "taxonomy": "iam",
        "field_title": translate(_("I am"), target_language=current_lang),
        "field_description": "",
        "default_language": "fr",
        "filename": "taxonomy-settings-iam.xml",
    }

    portal = api.portal.get()
    sm = portal.getSiteManager()
    theme_item = "collective.taxonomy.theme"
    district_item = "collective.taxonomy.district"
    iam_item = "collective.taxonomy.iam"
    utility_theme = sm.queryUtility(ITaxonomy, name=theme_item)
    utility_district = sm.queryUtility(ITaxonomy, name=district_item)
    utility_iam = sm.queryUtility(ITaxonomy, name=iam_item)

    if utility_theme and utility_district and utility_iam:
        return

    create_taxonomy_object(data_theme, portal)
    create_taxonomy_object(data_district, portal)
    create_taxonomy_object(data_iam, portal)

    # remove taxonomy test
    item = "collective.taxonomy.test"
    utility = sm.queryUtility(ITaxonomy, name=item)
    if utility:
        utility.unregisterBehavior()
        sm.unregisterUtility(utility, ITaxonomy, name=item)
        sm.unregisterUtility(utility, IVocabularyFactory, name=item)
        sm.unregisterUtility(utility, ITranslationDomain, name=item)


def create_taxonomy_object(data_tax, portal):
    taxonomy = registerTaxonomy(
        api.portal.get(),
        name=data_tax["taxonomy"],
        title=data_tax["field_title"],
        description=data_tax["field_description"],
        default_language=data_tax["default_language"],
    )

    adapter = TaxonomyImportExportAdapter(portal)
    data_path = os.path.join(os.path.dirname(__file__), "data")
    file_path = os.path.join(data_path, data_tax["filename"])
    with open(file_path, "r") as f:
        adapter.importDocument(taxonomy, f.read().encode("utf8"))

    del data_tax["taxonomy"]
    del data_tax["filename"]
    taxonomy.registerBehavior(**data_tax)


def add_behavior(type_name, behavior_name):
    """Add a behavior to a type"""
    fti = queryUtility(IDexterityFTI, name=type_name)
    if not fti:
        return
    behaviors = list(fti.behaviors)
    if behavior_name not in behaviors:
        behaviors.append(behavior_name)
    fti._updateProperty("behaviors", tuple(behaviors))


def _activate_faceted_navigation(context, configuration=False, path=None):
    subtyper = context.restrictedTraverse("@@faceted_subtyper")
    if subtyper.is_faceted:
        return
    subtyper.enable()
    if configuration and path:
        fpath = os.path.dirname(__file__) + path
        context.unrestrictedTraverse("@@faceted_exportimport").import_xml(
            import_file=open(fpath, "r").read().encode("utf8")
        )


def _disable_portlets(context, disabled=("plone.leftcolumn", "plone.rightcolumn")):
    for name in disabled:
        manager = getUtility(IPortletManager, name=name)
        blacklist = getMultiAdapter((context, manager), ILocalPortletAssignmentManager)
        blacklist.setBlacklistStatus(CONTEXT_CATEGORY, True)


def _publish(obj):
    if api.content.get_state(obj) != "published":
        api.content.transition(obj, transition="publish")


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.



# -*- coding: utf-8 -*-

from ideabox.policy import _
from plone.app.registry.browser import controlpanel
from zope import schema
from zope.interface import Interface


class InvalidEmailError(schema.ValidationError):
    __doc__ = u"Please enter a valid e-mail address."


class IIdeaBoxSettingsSchema(Interface):

    project_manager_email = schema.TextLine(
        title=_(u"Email address of the project manager"),
        description=_(
            u"If there are multiple email addresses, separate them with semicolons"
        ),
    )

    project_submission = schema.Bool(
        title=_(u"Enable / Disable project submission"), default=True
    )


class IdeaBoxSettingsEditForm(controlpanel.RegistryEditForm):

    schema = IIdeaBoxSettingsSchema
    label = _(u"Configuration for ideabox product")
    description = _(u"")

    def updateFields(self):
        super(IdeaBoxSettingsEditForm, self).updateFields()

    def updateWidgets(self):
        super(IdeaBoxSettingsEditForm, self).updateWidgets()


class IdeaBoxSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = IdeaBoxSettingsEditForm



# -*- coding: utf-8 -*-

from plone import api


def fix_comments_1001(context):
    from plone.app.discussion.interfaces import IConversation

    brains = api.content.find(
        context=api.portal.get(),
        total_comments={"query": [1, 1000], "range": "min:max"},
    )
    for brain in brains:
        obj = brain.getObject()
        conversation = IConversation(obj, None)
        for thread in conversation.getThreads():
            comment = thread["comment"]
            if comment.author_name == comment.author_email:
                user = api.user.get(username=comment.author_username)
                infos = [user.getProperty("first_name"), user.getProperty("last_name")]
                comment.author_name = " ".join([i for i in infos if i])


def to_1002(context):
    """
    Fix project_district index
    """
    catalog = api.portal.get_tool("portal_catalog")
    catalog.delIndex("project_district")
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile("profile-ideabox.policy:default", "catalog")

    for brain in api.content.find(portal_type="Project"):
        brain.getObject().reindexObject(idxs=["project_district"])


def to_1003(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile("profile-ideabox.policy:default", "viewlets")


def to_1004(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile("profile-ideabox.policy:default", "typeinfo")



# -*- coding: utf-8 -*-

from plone.dexterity.content import Container
from plone.supermodel import model
from zope.interface import implementer


class ICampaign(model.Schema):
    pass


@implementer(ICampaign)
class Campaign(Container):
    pass



# -*- coding: utf-8 -*-
from Products.CMFPlone.utils import getToolByName
from collective.z3cform.select2.widget.widget import MultiSelect2FieldWidget
from ideabox.policy import _
from plone import api
from plone.app.dexterity import _ as DXMF
from plone.app.textfield import RichText
from plone.app.textfield.value import IRichTextValue
from plone.app.z3cform.widget import RichTextFieldWidget
from plone.autoform import directives as form
from plone.dexterity.browser import view
from plone.dexterity.content import Container
from plone.indexer.decorator import indexer
from plone.supermodel import model
from six import text_type
from zope import schema
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory


class IProject(model.Schema):
    """IProject"""

    title = schema.TextLine(title=DXMF(u"label_title", default=u"Title"), required=True)

    form.widget(project_theme=MultiSelect2FieldWidget)
    project_theme = schema.List(
        title=_(u"Domain(s) concerned"),
        value_type=schema.Choice(
            title=_(u"Domain(s) concerned"), vocabulary=u"collective.taxonomy.theme"
        ),
        required=False,
    )

    form.widget(project_district=MultiSelect2FieldWidget)
    project_district = schema.List(
        title=_(u"District(s) concerned"),
        value_type=schema.Choice(
            title=_(u"District(s) concerned"),
            vocabulary=u"collective.taxonomy.district",
        ),
        required=False,
    )

    body = RichText(title=_(u"Content"), required=True)
    form.widget("body", RichTextFieldWidget)
    model.primary("body")

    form.mode(original_author="hidden")
    original_author = schema.TextLine(title=_(u"Original author"), required=False)


@implementer(IProject)
class Project(Container):
    pass


class ProjectView(view.DefaultView):

    _timeline_states = (
        "deposited",
        "project_analysis",
        "vote",
        "result_analysis",
        "study_in_progress",
        "in_progress",
        "realized",
    )
    _rating_states = ("vote", "result_analysis", "rejected")

    @property
    def can_edit(self):
        return api.user.has_permission(
            "cmf.ModifyPortalContent", obj=self.context, user=api.user.get_current()
        )

    @property
    def default_image(self):
        """Try to find the default image for the project, return `None` otherwise"""
        portal = api.portal.get()
        if "project_default_large.jpg" in portal:
            return portal["project_default_large.jpg"].absolute_url()

    @property
    def get_images_url(self):
        contents = self.context.listFolderContents(
            contentFilter={"portal_type": "Image"}
        )
        images_url = []
        for content in contents:
            images_url.append(content.absolute_url())
        if not images_url:
            default_image = self.default_image
            if default_image is not None:
                images_url.append(default_image)
        return images_url

    def get_news(self):
        return api.content.find(
            context=self.context,
            portal_type="News Item",
            sort_on="Date",
            sort_order="descending",
        )

    @property
    def review_state(self):
        return api.content.get_state(obj=self.context)

    @property
    def before_selected(self):
        return self.review_state in self._timeline_states[:4] + ("rejected",)

    @property
    def _workflow_history(self):
        history = [
            {
                "order": self._timeline_states.index(l.get("review_state")),
                "state": l.get("review_state"),
                "date": l.get("time"),
            }
            for l in [val for val in self.context.workflow_history.values()][0]
            if l.get("review_state") in self._timeline_states
        ]
        return sorted(history, key=lambda x: x["order"])

    @property
    def can_view_timeline(self):
        return False  # Temporarily fix as requested
        if self.review_state == "rejected":
            return False
        return self.review_state in self._timeline_states

    @property
    def can_view_rating(self):
        return self.review_state in self._rating_states

    @property
    def timeline_states(self):
        history = self._workflow_history
        current_state = self.review_state
        first_timeline_states = self._timeline_states[:4]
        second_timeline_states = self._timeline_states[4:]
        selected_states = (
            self.review_state in first_timeline_states
            and first_timeline_states
            or second_timeline_states
        )
        states = [
            {"state": e, "date": "", "class": u"unfinished"} for e in selected_states
        ]
        idx = (
            current_state in self._timeline_states
            and self._timeline_states.index(current_state)
            or 0
        )
        for line in history:
            # Ensure that next steps that were completed in the past is not
            # displayed
            if line["order"] > idx:
                break
            state = [s for s in states if s["state"] == line["state"]]
            if len(state) == 1:
                state[0]["date"] = line["date"]
                state[0]["class"] = u"finished"
        return states

    @property
    def anonymous(self):
        return api.user.is_anonymous()

    def creator(self):
        return getattr(self.context, "original_author", self.context.Creator())

    def author(self):
        return api.user.get(self.creator())

    def authorname(self):
        author = self.author()
        if author:
            infos = [author.getProperty("first_name"), author.getProperty("last_name")]
            return " ".join([i for i in infos if i])
        return author and author["fullname"] or self.creator()

    def get_project_theme(self):
        if not self.context.project_theme:
            return
        factory = getUtility(IVocabularyFactory, "collective.taxonomy.theme")
        vocabulary = factory(self.context)
        values = []
        for token in self.context.project_theme:
            try:
                values.append(
                    [
                        translate(
                            vocabulary.getTerm(token).title, context=self.context
                        ),
                        token,
                    ]
                ),
            except KeyError:
                continue
        site = api.portal.get().absolute_url()
        links = []
        for value in values:
            links.append(
                "<a href={0}/projets#b_start=0&c5={1}>{2}</a>".format(
                    site, value[1], value[0]
                )
            )
        return ", ".join(links)

    def get_project_district(self):
        if self.context.project_district:
            factory = getUtility(IVocabularyFactory, "collective.taxonomy.district")
            vocabulary = factory(self.context)
            values = []
            for token in self.context.project_district:
                try:
                    values.append(
                        translate(vocabulary.getTerm(token).title, context=self.context)
                    )
                except KeyError:
                    continue
            return ", ".join(values)

    def display_mac(self):
        if (
            self.context.aq_parent.portal_type == "campaign"
            and self.context.aq_parent.image
        ):
            return True
        return False

    def get_mac(self):
        if (
            self.context.aq_parent.portal_type == "campaign"
            and self.context.aq_parent.image
        ):
            return "{0}/@@images/image".format(self.context.aq_parent.absolute_url())
        return ""


@indexer(IProject)
def searchabletext_project(object, **kw):
    result = []

    fields = ["title", "description", "body", "original_author"]
    for field_name in fields:
        value = getattr(object, field_name, None)
        if type(value) is text_type:
            result.append(value)
        elif IRichTextValue.providedBy(value):
            transforms = getToolByName(object, "portal_transforms")
            text = (
                transforms.convertTo("text/plain", value.raw, mimetype=value.mimeType)
                .getData()
                .strip()
            )
            result.append(text)
        else:
            text = value
            if text:
                result.append(text)
    return " ".join(result)



# -*- coding: utf-8 -*-

from plone import api
from Products.CMFPlone.browser.syndication.adapters import DexterityItem


class ProjectItem(DexterityItem):
    def __init__(self, context, feed):
        self.image = self.get_image(context)
        super(ProjectItem, self).__init__(context, feed)

    @property
    def file(self):
        if self.image:
            return self.image.image

    def get_image(self, context):
        images = context.listFolderContents(contentFilter={"portal_type": "Image"})
        if images:
            return images[0]
        else:
            portal = api.portal.get()
            if "project_default_large.jpg" in portal:
                return portal["project_default_large.jpg"]

    @property
    def file_url(self):
        url = self.base_url
        fi = self.file
        if fi is not None:
            filename = fi.filename
            if filename:
                url += "/{0}".format(self.image.id)
        return url



# -*- coding: utf-8 -*-

from ideabox.policy.content.project import IProject
from ideabox.policy.utils import _activate_dashboard_navigation
from tempfile import mkstemp
from zope.globalrequest import getRequest
from plone.protect.auto import safeWrite

import os


def project_image_changed(obj, event):
    if hasattr(obj, "aq_parent") is False:
        return
    if IProject.providedBy(obj.aq_parent):
        obj.aq_parent.reindexObject(idxs=["project_picture"])


def project_added(obj, event):
    request = getRequest()
    safeWrite(obj, request)
    obj.allow_discussion = True


def campaign_added(obj, event):
    file = open(os.path.dirname(__file__) + "/../faceted/config/campaign.xml")
    xml = file.read()
    file.close()

    xml = xml.replace(
        '<property name="default">//REMPLACE//</property>',
        '<property name="default">{0}</property>'.format(
            "/{0}".format("/".join(obj.absolute_url_path().split("/")[2:]))
        ),
    )
    fd, path = mkstemp()
    with open(path, "w") as f:
        f.write(xml)
    os.close(fd)
    _activate_dashboard_navigation(obj, path)
    request = getRequest()
    request.response.redirect(obj.absolute_url())



# -*- coding: utf-8 -*-

from ideabox.policy.faceted import subscriber
from unittest import mock

import unittest


class TestRandomizer(unittest.TestCase):
    @mock.patch("time.time", mock.Mock(return_value=1546380000.0))
    def test_cache_key_basic(self):
        # 2019-01-01 23:00
        self.assertEqual(
            ("generator", 35795.0), subscriber._cache_key("function", "generator")
        )

    @mock.patch("time.time", mock.Mock(side_effect=(1546300740.0, 1546300800.0)))
    def test_cache_key_night_change(self):
        # 2019-01-01 00:59 and 2019-01-01 01:00
        self.assertEqual(
            ("generator", 35793.0), subscriber._cache_key("function", "generator")
        )
        self.assertEqual(
            ("generator", 35794.0), subscriber._cache_key("function", "generator")
        )

    @mock.patch("time.time", mock.Mock(side_effect=(1546343940.0, 1546344000.0)))
    def test_cache_key_noon_change(self):
        # 2019-01-01 12:59 and 2019-01-01 13:00
        self.assertEqual(
            ("generator", 35794.0), subscriber._cache_key("function", "generator")
        )
        self.assertEqual(
            ("generator", 35795.0), subscriber._cache_key("function", "generator")
        )

    def test_cache_key_generator(self):
        self.assertEqual(
            "0cc175b9c0f1b6a831c399e269772661".encode("utf8"),
            subscriber.random_key_generator(u"a"),
        )

    def test_random_sort(self):
        request = type("request", (object,), {"cookies": {"__ac": "a"}})()
        with mock.patch(
            "ideabox.policy.faceted.subscriber.random_key_generator", return_value="aa"
        ):
            randomizer = subscriber.Randomizer(request)
            self.assertEqual("project_random_1", randomizer.random_sort_key)
            self.assertEqual("descending", randomizer.random_sort_order)
        with mock.patch(
            "ideabox.policy.faceted.subscriber.random_key_generator", return_value="bb"
        ):
            randomizer = subscriber.Randomizer(request)
            self.assertEqual("project_random_2", randomizer.random_sort_key)
            self.assertEqual("ascending", randomizer.random_sort_order)
        with mock.patch(
            "ideabox.policy.faceted.subscriber.random_key_generator", return_value="ab"
        ):
            randomizer = subscriber.Randomizer(request)
            self.assertEqual("project_random_1", randomizer.random_sort_key)
            self.assertEqual("ascending", randomizer.random_sort_order)
        with mock.patch(
            "ideabox.policy.faceted.subscriber.random_key_generator", return_value="cc"
        ):
            randomizer = subscriber.Randomizer(request)
            self.assertEqual("project_random_3", randomizer.random_sort_key)
            self.assertEqual("descending", randomizer.random_sort_order)
        with mock.patch(
            "ideabox.policy.faceted.subscriber.random_key_generator", return_value="dd"
        ):
            randomizer = subscriber.Randomizer(request)
            self.assertEqual("project_random_4", randomizer.random_sort_key)
            self.assertEqual("ascending", randomizer.random_sort_order)
        with mock.patch(
            "ideabox.policy.faceted.subscriber.random_key_generator", return_value="ee"
        ):
            randomizer = subscriber.Randomizer(request)
            self.assertEqual("project_random_5", randomizer.random_sort_key)
            self.assertEqual("descending", randomizer.random_sort_order)
        with mock.patch(
            "ideabox.policy.faceted.subscriber.random_key_generator", return_value="ff"
        ):
            randomizer = subscriber.Randomizer(request)
            self.assertEqual("project_random_6", randomizer.random_sort_key)
            self.assertEqual("ascending", randomizer.random_sort_order)
        with mock.patch(
            "ideabox.policy.faceted.subscriber.random_key_generator", return_value="gg"
        ):
            randomizer = subscriber.Randomizer(request)
            self.assertEqual("project_random_7", randomizer.random_sort_key)
            self.assertEqual("descending", randomizer.random_sort_order)
        with mock.patch(
            "ideabox.policy.faceted.subscriber.random_key_generator", return_value="hh"
        ):
            randomizer = subscriber.Randomizer(request)
            self.assertEqual("UID", randomizer.random_sort_key)
            self.assertEqual("ascending", randomizer.random_sort_order)



# -*- coding: utf-8 -*-

from ideabox.policy.form.project_submission import ProjectSubmissionForm
from plone import api
from plone.z3cform.layout import FormWrapper


class ProjectSubmissionView(FormWrapper):
    form = ProjectSubmissionForm

    def enable_submission(self):
        return api.portal.get_registry_record(
            "ideabox.policy.browser.controlpanel.IIdeaBoxSettingsSchema.project_submission",
            default=True,
        )



# encoding: utf-8

from plone import api
from plone.i18n.normalizer import urlnormalizer
from transaction import commit
from Zope2.App import startup
from zope.component.hooks import setSite

import argparse
import csv
import password_generator


def add_user(author, mail):
    if len(author) < 3:
        author_id = urlnormalizer.normalize(mail[0:3].decode("utf8"), locale="fr")
    else:
        author_id = urlnormalizer.normalize(author.decode("utf8"), locale="fr")
    with api.env.adopt_user(username="admin"):
        if api.user.get(username=author_id) is None:
            pwd = password_generator.generate(length=20)
            user = api.user.create(
                username=author_id, email=mail, password="{0}1.".format(pwd)
            )
            user.setMemberProperties(mapping={"fullname": author})


def data_recovery(filename):
    csv_file = open(filename, "rb")
    reader = csv.reader(csv_file, delimiter=";")
    reader.next()
    for line in reader:
        author = line[3]
        author_mail = line[4]

        add_user(author, author_mail)


def main(app):
    startup.startup()

    parser = argparse.ArgumentParser()
    parser.add_argument("-c")
    parser.add_argument("csv", help="csv file")
    parser.add_argument("name", help="Name of plone site")
    args = parser.parse_args()

    setSite(app[args.name])

    if args.csv:
        data_recovery(args.csv)
        commit()


if "app" in locals():
    main(app)  # NOQA



# -*- coding: utf-8 -*-
from ideabox.policy import _
from plone import api
from plone.app.textfield import RichText
from plone.dexterity.content import Container
from plone.supermodel import model
from Products.Five import BrowserView
from zope import schema
from zope.interface import implementer


class IStateProgress(model.Schema):
    """IStateProgress"""

    state_date = schema.Date(title=_(u"Date"), required=True)

    body = RichText(title=_(u"Content"), required=True)


@implementer(IStateProgress)
class StateProgress(Container):
    pass


class StateProgressView(BrowserView):
    def get_state_progress(self):
        return api.content.find(
            context=self.context, portal_type="state_progress", sort_on="state_date"
        )



# encoding: utf-8

from Products.CMFPlone.utils import safe_unicode
from Zope2.App import startup
from cioppino.twothumbs import rate
from ideabox.policy.utils import token_type_recovery
from plone import api
from plone.i18n.normalizer import urlnormalizer
from plone.namedfile.file import NamedBlobImage
from transaction import commit
from zope.component.hooks import setSite

import argparse
import csv
import os


def add_project(
    container,
    project_id,
    title,
    project_theme,
    project_body,
    image_source,
    project_author,
    project_like,
    project_unlike,
    project_status,
):
    img_extensions = ("jpg", "png", "gif")
    with api.env.adopt_user(username="admin"):
        project = api.content.create(
            type="Project",
            title=title,
            project_theme=project_theme,
            body=u"<br>".join(project_body.decode("utf8").splitlines()),
            container=container,
            original_author=project_author,
            original_id=project_id,
        )
        for ext in img_extensions:
            filename = "{0}.{1}".format(project_id, ext)
            img_path = os.path.join(image_source, filename)
            if os.path.exists(img_path):
                add_image_from_file(project, filename, image_source)
                break

    rate.setupAnnotations(project)
    if project_like:
        for x in range(0, int(project_like)):
            rate.loveIt(project, "userimportp" + str(x))
    if project_unlike:
        for x in range(0, int(project_unlike)):
            rate.hateIt(project, "userimportn" + str(x))

    with api.env.adopt_user(username="admin"):
        if project_status in ["selected", "rejected"]:
            api.content.transition(obj=project, transition="deposit")
            api.content.transition(obj=project, transition="to_be_analysed")
            api.content.transition(obj=project, transition="vote")
            api.content.transition(obj=project, transition="vote_analysis")
            if project_status == "selected":
                api.content.transition(obj=project, transition="accept")
            else:
                api.content.transition(obj=project, transition="reject")


def add_image_from_file(container, file_name, source):
    file_path = os.path.join(source, file_name)
    if not container.hasObject(file_name):
        # with deterity image
        named_blob_image = NamedBlobImage(
            data=open(file_path, "r").read(), filename=safe_unicode(file_name)
        )
        image = api.content.create(
            type="Image",
            title=file_name,
            image=named_blob_image,
            container=container,
            language="fr",
        )
        image.setTitle(file_name)
        image.reindexObject()


def data_recovery(filename, image, container, status):
    csv_file = open(filename, "rb")
    reader = csv.reader(csv_file, delimiter=";")
    reader.next()
    for line in reader:
        project_id = line[0]
        project_title = line[2]
        project_theme = line[1]
        project_body = line[18]
        project_author = line[3]
        project_mail = line[4]
        project_like = line[15]
        project_unlike = line[16]

        token_type = token_type_recovery(project_theme)

        if len(project_author) < 3:
            project_author = urlnormalizer.normalize(
                project_mail[0:3].decode("utf8"), locale="fr"
            )
        else:
            project_author = urlnormalizer.normalize(
                project_author.decode("utf8"), locale="fr"
            )

        add_project(
            container,
            project_id,
            project_title,
            token_type,
            project_body,
            image,
            project_author,
            project_like,
            project_unlike,
            status,
        )


def follow_path(portal, path):
    element = portal
    for id in path.split("/"):
        if id not in element:
            raise ValueError("The id {0} does not exist".format(id))
        element = element[id]
    return element


def main(app):
    startup.startup()

    parser = argparse.ArgumentParser()
    parser.add_argument("-c")
    parser.add_argument("csv", help="csv file")
    parser.add_argument("img", help="localisation of pictures")
    parser.add_argument("name", help="Name of plone site")
    parser.add_argument("status", help="status of types")
    parser.add_argument("--path", dest="path", default="projets", help="creation path")
    args = parser.parse_args()

    setSite(app[args.name])
    portal = app[args.name]
    container = follow_path(portal, args.path)

    if args.csv and args.img:
        data_recovery(args.csv, args.img, container, args.status)
        commit()


if "app" in locals():
    main(app)  # NOQA



# -*- coding: utf-8 -*-

from collective.excelexport.datasources.base import BaseContentsDataSource
from datetime import datetime
from ideabox.policy.export.interfaces import IUsersExportable
from plone import api
from zope.component import adapts
from zope.component import getAdapters
from zope.interface import Interface


class UsersDataSource(BaseContentsDataSource):
    """
    Export Users
    """

    adapts(Interface, Interface)

    def get_filename(self):
        return "users-%s.xls" % (datetime.now().strftime("%d-%m-%Y"))

    def get_objects(self):
        return api.user.get_users()

    def get_sheets_data(self):
        data = []
        exportables = []
        exportables.extend(self.get_exportables())
        title = "Users"
        data.append(
            {"title": title, "objects": self.get_objects(), "exportables": exportables}
        )
        return data

    def get_exportables(self):
        users = [ad[1] for ad in getAdapters((self.context,), IUsersExportable)]
        return users



# -*- coding: utf-8 -*-

from collective.excelexport.interfaces import IExportable


class IExtendedProjectExportable(IExportable):
    """
    """


class IUsersExportable(IExportable):
    """
    """



# -*- coding: utf-8 -*-

from ideabox.policy.content.project import IProject
from plone.indexer import indexer

import hashlib


@indexer(IProject)
def project_picture(obj):
    images = obj.listFolderContents(contentFilter={"portal_type": "Image"})
    if images:
        obj.image = images[0].image
        return images[0].id


def random_hash(obj, method, r_min=0, r_max=32):
    me = getattr(hashlib, method)
    return me(obj.id.encode("utf8")).hexdigest()[r_min:r_max]


@indexer(IProject)
def project_random_1(obj):
    return random_hash(obj, "md5", r_min=0, r_max=32)


@indexer(IProject)
def project_random_2(obj):
    return random_hash(obj, "sha256", r_min=0, r_max=32)


@indexer(IProject)
def project_random_3(obj):
    return random_hash(obj, "sha256", r_min=32, r_max=64)


@indexer(IProject)
def project_random_4(obj):
    return random_hash(obj, "sha512", r_min=0, r_max=32)


@indexer(IProject)
def project_random_5(obj):
    return random_hash(obj, "sha512", r_min=32, r_max=64)


@indexer(IProject)
def project_random_6(obj):
    return random_hash(obj, "sha512", r_min=64, r_max=96)


@indexer(IProject)
def project_random_7(obj):
    return random_hash(obj, "sha512", r_min=96, r_max=128)



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from ideabox.policy.testing import IDEABOX_POLICY_INTEGRATION_TESTING
from Products.CMFPlone.utils import get_installer

import unittest


class TestSetup(unittest.TestCase):
    """Test that ideabox.policy is properly installed."""

    layer = IDEABOX_POLICY_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.installer = get_installer(self.portal, self.layer["request"])

    def test_product_installed(self):
        """Test if ideabox.policy is installed."""
        self.assertTrue(self.installer.isProductInstalled("ideabox.policy"))

    def test_browserlayer(self):
        """Test that IIdeaboxPolicyLayer is registered."""
        from ideabox.policy.interfaces import IIdeaboxPolicyLayer
        from plone.browserlayer import utils

        self.assertIn(IIdeaboxPolicyLayer, utils.registered_layers())


class TestUninstall(unittest.TestCase):

    layer = IDEABOX_POLICY_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.installer = get_installer(self.portal, self.layer["request"])
        self.installer.uninstallProducts(["ideabox.policy"])

    def test_product_uninstalled(self):
        """Test if ideabox.policy is cleanly uninstalled."""
        self.assertFalse(self.installer.isProductInstalled("ideabox.policy"))

    def test_browserlayer_removed(self):
        """Test that IIdeaboxPolicyLayer is removed."""
        from ideabox.policy.interfaces import IIdeaboxPolicyLayer
        from plone.browserlayer import utils

        self.assertNotIn(IIdeaboxPolicyLayer, utils.registered_layers())



# -*- coding: utf-8 -*-

from AccessControl import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import setSecurityManager
from AccessControl.User import Super as BaseUnrestrictedUser
from datetime import datetime
from eea.facetednavigation.layout.interfaces import IFacetedLayout
from ideabox.policy import vocabularies
from plone import api
from zope.i18n import translate

import os


def token_type_recovery(value):
    value = value.decode("utf8")
    vocabulary = vocabularies.ThemeVocabulary(None)
    return [
        e.token for e in vocabulary.by_value.values() if translate(e.title) == value
    ][0]


class UnrestrictedUser(BaseUnrestrictedUser):
    """Unrestricted user that still has an id.
    """

    def getId(self):
        """Return the ID of the user.
        """
        return self.getUserName()


def execute_under_admin(portal, function, *args, **kwargs):
    """ Execude code under admin privileges """
    sm = getSecurityManager()
    try:
        try:
            tmp_user = UnrestrictedUser("admin", "", [""], "")
            # Wrap the user in the acquisition context of the portal
            tmp_user = tmp_user.__of__(portal.acl_users)
            newSecurityManager(None, tmp_user)
            # Call the function
            return function(*args, **kwargs)
        except:
            # If special exception handlers are needed, run them here
            raise
    finally:
        # Restore the old security manager
        setSecurityManager(sm)


def review_state(context):
    return api.content.get_state(obj=context)


def can_view_rating(context):
    _rating_states = ("vote", "result_analysis", "rejected")
    return review_state(context) in _rating_states


def now():
    return datetime.now()


def _activate_dashboard_navigation(context, config_path=""):
    subtyper = context.restrictedTraverse("@@faceted_subtyper")
    if subtyper.is_faceted:
        return
    subtyper.enable()
    file = open(config_path, mode="rb")
    context.unrestrictedTraverse("@@faceted_exportimport").import_xml(import_file=file)
